<?php

namespace Drupal\snippet_manager\Plugin\SnippetVariable;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Retrieves plugin definitions for all entity form types.
 */
class EntityFormDeriver extends DeriverBase implements ContainerDeriverInterface {

  /**
   * Entity type manager.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs EntityDeriver object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type) {
      // Comment entity form has no sense when rendered standalone because it
      // needs a reference on parent entity.
      // Some entity types (i.e. File) does not have form classes.
      if ($entity_type instanceof ContentEntityTypeInterface && $entity_type_id != 'comment' && $entity_type->hasFormClasses()) {
          $this->derivatives[$entity_type_id] = $base_plugin_definition;
          $this->derivatives[$entity_type_id]['title'] = $entity_type->getLabel();
      }
    }
    return $this->derivatives;
  }

}
